@@ -41,6 +41,7 @@ class adam_config():
 
 
 
+         
         ##############################################################################################
 
 
@@ -63,7 +64,12 @@ class adam_config():
                       'prod1':     {'path_data'       : '/PROJETS/3895-ADAM/code/data/',
                                     'output_root_dir' : '/home/www/data/adam/web-root/web-output/',
                                     'debug_log_file'  : '/home/www/data/adam/wsgi-apps/wsgi-log.log',
-                                    'http_root'       : 'http://www.noveltis.com/adam-demo/web-output/'
+                                    'http_root'       : 'http://adam.noveltis.fr/web-output/'
+                                   },
+                      'adam-srv':     {'path_data'       : '/PROJETS/3895-ADAM/code/data/',
+                                    'output_root_dir' : '/home/www/data/adam/web-root/web-output/',
+                                    'debug_log_file'  : '/home/www/data/adam/wsgi-apps/wsgi-log.log',
+                                    'http_root'       : 'http://adam.noveltis.fr/web-output/'
                                    },
                       'cedric':    {'path_data'       : '/home/satellites9/cbacour/ADAM/Data/V1/',
                                     'output_root_dir' : '/home/satellites9/cbacour/ADAM/OUTPUT/',
@@ -97,18 +103,12 @@ class adam_config():
         self.file_ocean_WaterRefInd = 'ADAM_WaterRefractiveIndex_300-4000.txt'
         # absorption coefficient of water
         # from  D. J. Segelstein, "The complex refractive index of water," University of Missouri-Kansas City, (1981). (http://omlc.ogi.edu/spectra/water/data/segelstein81.dat)
-        self.file_ocean_WaterAbs = 'ADAM_WaterAbsorption_300-4000.txt' # CHANGE_CB
+        self.file_ocean_WaterAbs = 'ADAM_WaterAbsorption_300-4000.txt'
 
 
-
-        #OUTPUT_ROOT_DIR = '/home/satellites9/cbacour/ADAM/OUTPUT/'
-        #DEBUG_LOG_FILE  = '/home/satellites9/cbacour/ADAM/OUTPUT/'
-        #HTTP_ROOT       = '/home/satellites9/cbacour/ADAM/OUTPUT/'
-
         self.netcdf_datafile = os.path.join(self.path_data, self.netcdf_datafile)
         self.netcdf_file_covarland_template = os.path.join(self.path_data, self.netcdf_file_covarland_template)
 
-
         logging.basicConfig(filename=self.debug_log_file, level=logging.DEBUG, format='%(asctime)s %(levelname)s %(message)s')
 
 
@@ -116,16 +116,88 @@ class adam_config():
     # - Global parameters
     # -
 
+        # limits of reflectance for the various wavelengths used to define the colour scales
+        # for maps generated for the ADAM website.
+        # the spectral range is the key, from which reasonable max ranges can be had for global maps.
+        # the incrementing numbers (0->11) are the indexes used by the make_maps scripts, and
+        # correspond to the names of the images used by the website.
+        
+        self.reflectance_limits = {
+            # averaged 0
+            (300, 400):   {'land':  {'min': 0, 'max': 1.02},
+                           'ocean': {'min': 0, 'max': 0.10442} },
+            # averaged 1
+            (400, 700):   {'land':  {'min': 0, 'max': 1.02 },
+                           'ocean': {'min': 0, 'max': 0.0262 } },
+            # averaged 2
+            (700, 1300):  {'land':  {'min': 0, 'max': 0.928},
+                           'ocean': {'min': 0, 'max': 0.00044} },
+            # averaged 3
+            (1300, 2500): {'land':  {'min': 0, 'max': 0.69},
+                           'ocean': {'min': 0, 'max': 0.0001} },
+            # averaged 4
+            (2500, 4000): {'land':  {'min': 0, 'max': 0.55},
+                           'ocean': {'min': 0, 'max': 0.0001} },
+            # MODIS iii ## 5
+            (469,469):    {'land':  {'min': 0, 'max': 1.027 },
+                           'ocean': {'min': 0, 'max': 0.0512 } },
+            # MODIS iv  ## 6
+            (555,555):    {'land':  {'min': 0, 'max': 1.023},
+                           'ocean': {'min': 0, 'max': 0.0222} },
+            # MODIS i   ## 7
+            (645,645):    {'land':  {'min': 0, 'max': 1.01 },
+                           'ocean': {'min': 0, 'max': 0.0082 } },
+            # MODIS ii  ## 8
+            (858,858):    {'land':  {'min': 0, 'max': 0.947},
+                           'ocean': {'min': 0, 'max': 0.0001} },
+            # MODIS v   ## 9
+            (1240,1240):  {'land':  {'min': 0, 'max': 0.621 },
+                          'ocean':  {'min': 0, 'max': 0.0001 } },
+            # MODIS vi  ## 10
+            (1640,1640):  {'land':  {'min': 0, 'max': 0.511},
+                           'ocean': {'min': 0, 'max': 0.0001} },
+            # MODIS vii ## 11
+            (2130,2130):  {'land':  {'min': 0, 'max': 0.493 },
+                           'ocean': {'min': 0, 'max': 0.0001 } }
+        }
+         
+        # arrays of the wavebands, each element either length=1 for narrow band 
+        # or length=2 to use spectral averaging.
+        self.instrument_wavebands = {
+            'modis'           : [469, 555, 645, 858, 1240, 1640, 2130],
+            'vgt'             : [450, 645, 840, 1665],
+            'avhrr'           : [635, 862, 1610, 3740],
+            'meris'           : [413, 443, 490, 510, 560, 620, 665, 681, 705, 754, 760, 775, 865, 885, 900],
+            'misr'            : [446, 558, 672, 867],
+            'seawifs  '       : [412, 443, 490, 510, 555, 670, 765, 865],
+            'polder_parasol'  : [445, 492, 565, 670, 763, 763, 861, 908],
+            'msg_seviri'      : [600, 800, 1600, 3900],
+            'sentinel3_olci'  : [413, 443, 490, 510, 555, 560, 620, 659, 665, 682, 709, 754, 761, 779, 865, 885, 900, 940, 1020, 1376, 1610, 2250, 3740],
+            'omi'             : [309, 312, 320, 328, 335, 343, 345, 354, 367, 373, 376, 380, 388, 406, 416, 418, 425, 440, 442, 452, 463, 471, 477, 488, 494, 499],
+            'orbview_2'       : [412, 443, 490, 510, 555, 670, 765, 865],
+            'sentinel2'       : [443, 490, 560, 665, 705, 741, 783, 842, 865, 945, 1375, 1610, 2190],
+            'aster'           : [560, 660, 810, 1650, 2165, 2205, 2260, 2330, 2395],
+            'spot_5'          : [545, 645, 835, 1665],
+            'spot_67'         : [490, 560, 660, 825],
+            'landsat_7'       : [485, 560, 660, 835, 1650, 2220],
+            'landsat_8'       : [443, 482, 562, 655, 865, 1375, 1610, 2200]
+        }
+        
+        # the gome instrument is every integer value between (and including) 240 and 790
+        # actually, adam has a minimum of 300 so we set to 300 here
+        self.instrument_wavebands['gome'] = range(300,791) 
+        
+
         # the maximum allowable size in degrees for a (non-download) request (20x20)
         self.limit_square_degrees = 410
 
-        #maximum limit to perform pixel-level calculations
+        # maximum limit to perform pixel-level calculations
         self.limit_degrees_pixel_calcs = 0.1
 
         # -
         # - Names of the variables to read in the gridded ADAM NetCDF file
         # -
-        self.vars_land = ['ref_land']   # respect this order!
+        self.vars_land = ['ref_land']                       # respect this order!
         self.vars_ocean = ['wind_speed','chloro_conc']      # respect this order!
 
 
@@ -134,16 +206,6 @@ class adam_config():
         self.vza_std = 0
         self.phi_std = 0
 
-        vza_range  = [-65,65]
-        vza_step   = 5
-        self.vza_values = np.arange(vza_range[0], vza_range[1]+5, vza_step)
-        self.phi_values = np.arange(0,360+10,10)
-
-        self.step_polar = 41 # discretisation step for drawing polar plots
-
-        # standard BRDF F1 & F2 functions over land (in the standard observation geometry)
-        self.f1_std = process_brdf.brdf_land_f1(self.sza_std,self.vza_std,self.vza_std)
-        self.f2_std = process_brdf.brdf_land_f2(self.sza_std,self.vza_std,self.vza_std)
 
         # maximum value authorized
         self.sza_max_limit = 70
@@ -155,6 +217,17 @@ class adam_config():
         self.beyond_za_limit = 'constant'
 
 
+        vza_range  = [0,self.vza_max_limit]
+        vza_step   = 5
+        self.vza_values = np.arange(-vza_range[1], vza_range[1]+5, vza_step)
+        self.phi_values = np.arange(0,360+10,10)
+        self.step_polar = 41 # discretisation step for drawing polar plots
+
+        # standard BRDF F1 & F2 functions over land (in the standard observation geometry)
+        self.f1_std = process_brdf.brdf_land_f1(self.sza_std,self.vza_std,self.vza_std)
+        self.f2_std = process_brdf.brdf_land_f2(self.sza_std,self.vza_std,self.vza_std)
+
+
         # missing values
         self.missval_1b = 255
         self.missval_2b = -9999.
@@ -180,7 +253,7 @@ class adam_config():
             raise Exception('Error opening main source data file, location given was: %s' % self.netcdf_datafile)
 
 
-        # load the modis wavebands for convenience later
+        # load the modis wavebands
         self.modis_wavebands = np.round(source_dataset.variables['wavebands'][:])
         source_dataset.close()
         self.indices_modis_bands = []
@@ -213,7 +286,6 @@ class adam_config():
 
             # - Spectrum parameters (ACP)
             'file_acp'            : file_acp,
-            #'ACP_eigenvectors'    : dataACP.variables['Eigenvectors'][:, dataACP.variables['neigenvectors'][:]],
             'ACP_ObsMean'         : dataACP.variables['ObsMean'][:],
             'ACP_SpecMean'        : dataACP.variables['SpecMean'][:],
             'ACP_prod'            : dataACP.variables['prodACP'][:],
@@ -233,7 +305,7 @@ class adam_config():
 
             # - snow
             'snow_grain_size'         : 0.12e-3,
-            'chanB'                   : 0, #CHANGE_CB
+            'chanB'                   : 0,
             'chanR'                   : 2,
             'chanPIR'                 : 3,
             'chanPIR2'                : 4,
@@ -249,8 +321,6 @@ class adam_config():
 
         # this consumes ~5 seconds !
         config_land['ACP_eigenvectors'] = dataACP.variables['Eigenvectors'][:, config_land['ACP_neigenvectors']]
-        #config_land['ACP_eigenvectors'] = config_land['ACP_eigenvectors'][:, config_land['ACP_neigenvectors']][:]
-
         config_land['ACP_mat'] = np.dot(config_land['ACP_eigenvectors'], config_land['ACP_prod'])
 
         dataACP.close()
@@ -300,14 +370,14 @@ class adam_config():
         config_ocean['ref_chl_std'] = ref_ocean_chl_std
         config_ocean['lmbd_chl_std'] = lmbd_ocean_chl_std
 
-        # > MODIF_CB
         # - foam reflectance as a function of foam (from Koepke, 1984)
         data = np.array([[4,  0.00],[5,  0.00],[6,  0.00],[7,  0.00], [8,  0.10],[9,  0.20],\
                          [10, 0.20],[11, 0.30],[12, 0.40],[13, 0.50],[14, 0.70],[15, 0.90],\
                          [16, 1.10],[17, 1.30],[18, 1.60],[19, 2.00],[20, 2.30],[25, 5.10]])
         ws_0 = data[:,0].ravel()
         ref_0 = (data[:,1].ravel())/100.
-        #  interpolation for wind speed between [0 ; 30] m.s-1
+
+        # - interpolation for wind speed between [0 ; 30] m.s-1
         ws_foam_std = np.arange(31)
         ref_foam_std = np.interp(ws_foam_std, ws_0, ref_0)
         config_ocean['foam_ws_std'] = ws_foam_std
@@ -325,7 +395,6 @@ class adam_config():
         ilmbd_VIS = np.where(self.lmbd == 400)[0][0]
         config_ocean['foam_normfac'] = config_ocean['foam_specmod'][ilmbd_VIS]
 
-        # < MODIF_CB
 
     # -
     # - Return
