@@ -17,23 +17,26 @@ import process_reflectance
 #
 def main(reflectance, data, cfg, sza, lmbd, vza, phi,
          sza_max_limit=None, vza_max_limit=None, beyond_za_limit=None,
-         do_spectral_averaging=None, do_compute_error=False):
-    ''' Compute the BRDF in the prescribed observation geometry.
-
-         Inputs
-          - reflectance array [nlon, nlat, nlmbd]
-          - wind speed [nlon, nlat]
-          - wind speed [nlon, nlat]
-          - cfg parameters
-          - sun zenith angle (optional):       scalar
-          - view zenith angle (optional):      scalar or vector
-          - relative azimuth angle (optional): scalar or vector
-
-         Outputs
-          - reflectance array which size depends on the number of view directions
-            - [nlon, nlat, nlmbd] if vza is a scalar
-            - [nlon, nlat, nlmbd, nang] if vza is a vector
+         do_spectral_averaging=None, do_compute_error=True):
+    '''
+    Compute the BRDF in the prescribed observation geometry, with a specific treatment
+    depending on the surface (land, snow, or ocean):
+    - Land: Ross-Li HS BRDF model (+ associated uncertainty)
+    - Snow: Kokhanovsky BRDF model
+    - Ocean: Breon-Henriot (from Cox and Munk) Glint model + contribution of the foam
 
+    Inputs:
+    - reflectance:          array [nlon, nlat, nlmbd]
+    - wind speed:           array [nlon, nlat]
+    - cfg parameters
+    - sun zenith angle (optional):       scalar
+    - view zenith angle (optional):      scalar or array [nvza]
+    - relative azimuth angle (optional): scalar or array [nphi]
+    
+    Outputs:
+    - List of 2 elements:
+    - ref_ans:       array [nlon, nlat, nlmbd, nvza, nphi]
+    - ref_err_ans:   array [nlon, nlat, nlmbd, nvza, nphi]
     '''
 
     if sza_max_limit is None  : sza_max_limit = cfg.sza_max_limit
@@ -55,8 +58,6 @@ def main(reflectance, data, cfg, sza, lmbd, vza, phi,
 
 
     ref_ans = np.zeros((npix_X * npix_Y, nlmbd, len(vza), len(phi)), np.float32) + cfg.missval_2b
-
-    # > MODIF_CB
     ref_err_ans = np.empty(ref_ans.shape)
     ref_err_ans[:] = np.NaN
 
@@ -86,15 +87,21 @@ def main(reflectance, data, cfg, sza, lmbd, vza, phi,
     # - Land processing
     # -
     if len(data['idx_land']) > 0:
-        idx_land = data['idx_land']
 
+        # The current configuration assumes that SZA is the same for all land pixels.
+        # Therefore the kernels need to be computed only once.
+        # If SZA should vary with land point => comment the initialisation lines in function_brdf_land
+        kernels = function_init_kernels_land()
+
+        idx_land = data['idx_land']
         ipix = 0
         for idx in idx_land:
-            ans = function_brdf_land(ref[idx,:], ndvi[idx], cfg, sza=sza, vza=vza, phi=phi)
-
+            ans = function_brdf_land(ref[idx,:], ndvi[idx], cfg, kernels,
+                                     sza=sza, vza=vza, phi=phi)
             if do_compute_error:
                 ans_err =  function_brdf_land_error(ref[idx,:],
                                                     cfg,
+                                                    kernels,
                                                     sza=sza,
                                                     vza=vza,
                                                     phi=phi,
@@ -103,7 +110,6 @@ def main(reflectance, data, cfg, sza, lmbd, vza, phi,
                                                     phi_std=cfg.phi_std)
 
 
-
             ref_ans[idx,:,:,:] = np.reshape(ans,(1, nlmbd, vza.shape[0], phi.shape[0]))
             if do_compute_error:
                 ref_err_ans[idx,:,:,:] = np.reshape(ans_err,(1, nlmbd, vza.shape[0], phi.shape[0]))
@@ -143,10 +149,16 @@ def main(reflectance, data, cfg, sza, lmbd, vza, phi,
             for iphi in range(len(phi)):
                 ref_foam[:,:,ivza,iphi] = ans_foam
 
+        # The current configuration assumes that SZA is the same for all land pixels.
+        # Therefore the directional "parameters" need to be computed only once.
+        # If SZA should vary with land point => comment the initialisation lines in function_brdf_ocean
+        vars_ocean = function_init_dir_ocean()
+
         for iteration, idx in enumerate(data['idx_ocean']):
             ans = function_brdf_ocean(  ref[idx,:],
                                         ws[idx],
                                         cfg,
+                                        vars_ocean,
                                         sza=sza,
                                         vza=vza,
                                         phi=phi,
@@ -172,7 +184,7 @@ def main(reflectance, data, cfg, sza, lmbd, vza, phi,
             ref_ans[:,:,idxBAD,:] = 0
             ref_err_ans[:,:,idxBAD,:] = 0
         if beyond_za_limit == 'nan':
-            ref_ans[:,:,idxBAD,:] = np.nan # IVAN / or cfg.missval ???
+            ref_ans[:,:,idxBAD,:] = np.nan
             ref_err_ans[:,:,idxBAD,:] = np.nan
 
 
@@ -181,53 +193,81 @@ def main(reflectance, data, cfg, sza, lmbd, vza, phi,
     ans[1] = ref_err_ans
     return ans
 
-# END compute
+# END main
 # ===================================================================
 
 
-
-
-
 # ===================================================================
-# Compute BRDF over Land on a pixel basis from
+# Initialise the kernels dictionnary
 #
-def function_brdf_land(ref, ndvi, cfg, sza=None, vza=None, phi=None):
-    '''Compute BRDF over Land on a pixel basis from
+# The current configuration assumes that SZA is the same for all land pixels.
+# Therefore the kernels need to be computed only once.
+# If SZA should vary with land point => comment the initialisation lines in function_brdf_land
+def function_init_kernels_land():
+    '''
+    Initialisation of the F1 and F2 kernels of the Ross-Li-Maignan
+    BRDF model.
+    The current configuration assumes that SZA is the same for all land pixels.
+    Therefore the kernels will be computed only once.
+    If SZA should vary with land point => comment the initialisation lines in function_brdf_land
+    '''
+    kernels = {}
+    kernels['f1'] = None
+    kernels['f2'] = None
+    return kernels
+# END function_init_kernels_land
+# ===================================================================
 
 
+# ===================================================================
+# Compute BRDF over Land on a pixel basis
+#
+def function_brdf_land(ref, ndvi, cfg, kernels, sza=None, vza=None, phi=None):
+    '''
+    Compute BRDF over Land on a pixel basis from the reflectance values on
+    the standard observation geometry and the Ross-Li-Maignan BRDF model, the
+    directional parameters of which are scaled as a function of NDVI
+
          Inputs
-          - spectral reflectance   scalar or array [nlmbd]
-          - NDVI
+          - spectral reflectance  ref:         array [nlmbd]
+          - ndvi:                              scalar
+          - kernels classes including:
+             - f1:                             array [nvza*nphi]
+             - f2:                             array [nvza*nphi]
+          - cfg parameters
           - sun zenith angle (optional):       scalar
-          - view zenith angle (optional):      scalar or vector
-          - relative azimuth angle (optional): scalar or vector
+          - view zenith angle (optional):      scalar or array [nvza]
+          - relative azimuth angle (optional): scalar or array [nphi]
 
          Outputs
-          - brdf array of size (depending on the cases)
-            - 1
-            - nlmbd
-            - nang
-
+          - brdf:                              array [nlmbd, nvza, nphi]
     '''
     if sza is None: sza = cfg.sza_std
     if vza is None: vza = cfg.vza_std
     if phi is None: phi = cfg.phi_std
 
-    # syncronise shapes, this is neccessary because the shape of vza and phi need to always be the same
-    #vza_meshed, phi_meshed = vza, phi
-    #if vza.shape != phi.shape:
-    phi_meshed, vza_meshed = np.meshgrid(phi, vza)
-    vza_meshed = vza_meshed.ravel()               
-    phi_meshed = phi_meshed.ravel()               
+    # - kernel Initialisation
+    # As SZA is the same for all pixels, do it only once.
+    # In case SZA should vary with the land point, then need to apply the following lines at each
+    # call of function_brdf_land
+    if kernels['f1'] ==  None:
+        # syncronise shapes, this is neccessary because the shape of vza and phi need to always be the same
+        phi_meshed, vza_meshed = np.meshgrid(phi, vza)
+        vza_meshed = vza_meshed.ravel()
+        phi_meshed = phi_meshed.ravel()
 
+        # kernels fonctions
+        f1 = brdf_land_f1(sza,vza_meshed,phi_meshed)
+        f2 = brdf_land_f2(sza,vza_meshed,phi_meshed)
 
-    # BRDF fonctions
-    f1 = brdf_land_f1(sza,vza_meshed,phi_meshed)
-    f2 = brdf_land_f2(sza,vza_meshed,phi_meshed)
+        kernels['f1']=f1
+        kernels['f2']=f2
 
+    else:
+        f1 = kernels['f1']
+        f2 = kernels['f2']
 
-    # Scaling of the R and V coefficients as a function of NDVI
-    #ndvi = (ref_MODIS[Cland.chanPIR]-ref_MODIS[Cland.chanR]) / (ref_MODIS[Cland.chanPIR]+ref_MODIS[Cland.chanR])
+    # - Scaling of the R and V coefficients as a function of NDVI
     Rr   = cfg.land.slope_R[0]*ndvi + cfg.land.intercept_R[0]
     Rnir = cfg.land.slope_R[1]*ndvi + cfg.land.intercept_R[1]
     Vr   = cfg.land.slope_V[0]*ndvi + cfg.land.intercept_V[0]
@@ -236,20 +276,19 @@ def function_brdf_land(ref, ndvi, cfg, sza=None, vza=None, phi=None):
     R = cfg.land.alpha_RV * Rnir + (1-cfg.land.alpha_RV)*Rr
     V = cfg.land.alpha_RV * Vnir + (1-cfg.land.alpha_RV)*Vr
 
-    # is it possible to provide commentry as to when each condition
-    # would be the case ? and the same for the snow and ocean ?
-    if f1.shape == () or ref.shape == (): # is this to determine if it is a float and not
-                                          # an array ? when would that happen ? why not
-                                          # use == type(float) ?
+    # Case where a float is returned
+    if f1.shape == () or ref.shape == ():
         brdf = ref*(1+R*f1+V*f2)/(1+R*cfg.f1_std+V*cfg.f2_std)
+    # Case where an array is returned
     else:
         brdf = np.zeros( (ref.shape[0], len(vza), len(phi) ),np.float32)
+        # case where len(lmbd) > len(angles) => loop on angles
         if ref.shape[0] > f1.shape[0]:
             for i in range(f1.shape[0]):
                 calc = ref[:]*(1+R*f1[i]+V*f2[i])/((1+R*cfg.f1_std+V*cfg.f2_std))
                 brdf[:,i] = calc.reshape(brdf[:,i].shape)
         else:
-            # for each waveband (?)
+            # case where len(lmbd) < len(angles) => loop on wavebands
             for i in range(ref.shape[0]):
                 calc = ref[i]*(1+R*f1[:]+V*f2[:])/(1+R*cfg.f1_std+V*cfg.f2_std)
                 brdf[i,:] = calc.reshape(brdf[i,:].shape)
@@ -263,23 +302,28 @@ def function_brdf_land(ref, ndvi, cfg, sza=None, vza=None, phi=None):
 # ===================================================================
 # Compute tne error on BRDF over Land on a pixel basis from
 #
-def function_brdf_land_error(ref, cfg, sza=None, vza=None, phi=None,
-                 sza_std=None, vza_std=None, phi_std=None):
-    '''Compute the error on BRDF over Land on a pixel basis from
-
+def function_brdf_land_error(ref, cfg, kernels,
+                             sza=None, vza=None, phi=None,
+                             sza_std=None, vza_std=None, phi_std=None):
+    '''
+    Compute the error on BRDF over Land on a pixel basis from the reflectance values on
+    the standard observation geometry and the Ross-Li-Maignan BRDF model
 
          Inputs
-          - spectral reflectance   scalar or array [nlmbd]
-          - sun zenith angle (optional):       scalar
-          - view zenith angle (optional):      scalar or vector
-          - relative azimuth angle (optional): scalar or vector
+          - spectral reflectance  ref:                  array [nlmbd]
+          - cfg parameters
+          - kernels classes including:
+             - f1:                                      array [nvza*nphi]
+             - f2:                                      array [nvza*nphi]
+          - sun zenith angle (optional):                scalar
+          - view zenith angle (optional):               scalar or array [nvza]
+          - relative azimuth angle (optional):          scalar or array [nphi]
+          - standard sun zenith angle (optional):       scalar
+          - standard view zenith angle (optional):      scalar
+          - standard relative azimuth angle (optional): scalar
 
          Outputs
-          - brdf array of size (depending on the cases)
-            - 1
-            - nlmbd
-            - nang
-
+          - err_brdf:                                   array [nlmbd, nvza, nphi]
     '''
     if sza is None: sza = cfg.sza_std
     if vza is None: vza = cfg.vza_std
@@ -288,10 +332,11 @@ def function_brdf_land_error(ref, cfg, sza=None, vza=None, phi=None,
     if vza_std is None: vza_std = cfg.vza_std
     if phi_std is None: phi_std = cfg.phi_std
 
+    f1 = kernels['f1']
+    f2 = kernels['f2']
+
     # syncronise shapes, this is neccessary because the shape of vza and phi need
     # to always be the same
-    #vza_meshed, phi_meshed = vza, phi
-    #if vza.shape != phi.shape:
     phi_meshed, vza_meshed = np.meshgrid(phi, vza)
     vza_meshed = vza_meshed.ravel()
     phi_meshed = phi_meshed.ravel()
@@ -308,127 +353,159 @@ def function_brdf_land_error(ref, cfg, sza=None, vza=None, phi=None,
         calc = np.sqrt(var_brdf)
         err_brdf[i,:] = calc.reshape(err_brdf[i,:].shape)
 
-
-    # when is this used ?
+    # Case where a float is returned
     if not f1.shape: err_brdf=np.reshape(err_brdf,ref.shape[0])
 
     # return value
     return err_brdf
-# END function_brdf_land
+# END function_brdf_land_error
 # ===================================================================
 
 
-
 # ===================================================================
-# Compute BRDF over Ocean on a pixel basis from the Cox and Munk model
+# Initialisation of the directional variables of the ocean Glint model
 #
-def function_brdf_ocean(ref, wind_speed, cfg, sza=None, vza=None, phi=None, lmbd = None,
+    '''
+    Initialisation of the directional variables of the ocean Glint model
+    The current configuration assumes that SZA is the same for all pixels,
+    therefore the directional variables will be computed only once.
+    '''
+
+def function_init_dir_ocean():
+    vars_ocean = {}
+    vars_ocean['ref_sp'] = None
+    vars_ocean['tan_beta'] = None
+    vars_ocean['beta'] = None
+    vars_ocean['cos_vza'] = None
+    return vars_ocean
+# END function_init_dir_ocean
+# ===================================================================
+
+
+
+# ===================================================================
+# Compute BRDF over Ocean on a pixel basis from the Breon-Henriot model
+# (from Cox and Munk)
+#
+def function_brdf_ocean(ref, wind_speed, cfg, vars_ocean, sza=None, vza=None, phi=None, lmbd = None,
                         multi_dir = False, do_spectral_averaging = None):
     '''
-    Compute BRDF over Oean on a pixel basis from the Cox and Munk model
+    Compute BRDF over Oean on a pixel basis from the breon-Henriot model (Cox and Munk)
     See Breon FM (1993), An analytical model for the Cloud-Free Atmosphere/Ocean
     system reflectance, Remote Sensing of Environment, Remote Sensing of Environment,
     43:179-192
     The specular reflectance is computed thanks to Fresnel equation, using the
     real parti of the water refractive index. The latter comes from http://refractiveindex.info/?group=LIQUIDS&material=Water /
-    G. M. Hale and M. R. Querry. Optical Constants of Water in the 200-nm to 200-microm Wavelength Region, Appl. Opt. 12, 555-563 (1973) doi:10.1364/AO.12.000555
+    G. M. Hale and M. R. Querry. Optical Constants of Water in the 200-nm to 200-microm Wavelength Region, Appl. Opt. 12, 555-563
+    (1973) doi:10.1364/AO.12.000555
 
     Inputs
-        - spectral reflectance
-        - wind_speed [npixels]
-        - sun zenith angle       :  scalar
-        - view zenith angle      : scalar or vector
-        - relative azimuth angle : scalar or vector
-        - wavelength             : scalar or vector
-        - wavelength for nwater  : vector
-        - water refraction index : vector
-        - multi_dir              : logical
+        - spectral reflectance ref:  array [nlmbd]
+        - wind_speed:                scalar
+        - cfg parameters
+        - vars_ocean:               arrays [nvza*nphi]
+        - sun zenith angle:         scalar
+        - view zenith angle:        scalar or array [nvza]
+        - relative azimuth angle:   scalar or array [nphi]
+        - lmbd:                     array [nlmbd]
+        - multi_dir:                logical
+        - do_spectral_averaging:    logical
 
 
     Outputs
-        - reflectance [nlmbd, nvza]
-
-
+        - brdf:                     array [nlmbd, nvza, nphi]
     '''
 
     if sza is None: sza = cfg.sza_std
     if vza is None: vza = cfg.vza_std
     if phi is None: phi = cfg.phi_std
 
-    # syncronise shapes, this is neccessary because the shape of vza and phi need to always be the same
-    #vza_meshed, phi_meshed = vza, phi
-    #if vza.shape != phi.shape:
-    phi_meshed, vza_meshed = np.meshgrid(phi, vza)
-    vza_meshed = vza_meshed.ravel()
-    phi_meshed = phi_meshed.ravel()
-
     dr = np.pi/180.
-
     ws = wind_speed
-
-    # - Specular reflectance
     nlmbd = len(lmbd)
-    nwater = cfg.ocean.nwater
 
-    # spectral subsampling
-    if nlmbd != len(cfg.lmbd):
-        idxWave = []
-        for wl in lmbd:
-            ind = np.ma.nonzero(cfg.ocean.lmbd_nwater == wl)[0]
-            idxWave.extend(ind.tolist())
-        nwater = cfg.ocean.nwater[idxWave]
+    # - Initialisation of the directional parameters
+    # As SZA is the same for all pixels, do it only once.
+    # In case SZA should vary with the land point, then need to apply the following lines at each
+    # call of function_brdf_ocean
+    if vars_ocean['ref_sp'] ==  None:
+        # syncronise shapes, this is neccessary because the shape of vza and phi need to always be the same
+        phi_meshed, vza_meshed = np.meshgrid(phi, vza)
+        vza_meshed = vza_meshed.ravel()
+        phi_meshed = phi_meshed.ravel()
+
+        cos_vza = np.cos(vza_meshed*dr)
+
+        # - Specular reflectance
+        nwater = cfg.ocean.nwater
+        # spectral subsampling
+        if nlmbd != len(cfg.lmbd):
+            idxWave = []
+            for wl in lmbd:
+                ind = np.ma.nonzero(cfg.ocean.lmbd_nwater == wl)[0]
+                idxWave.extend(ind.tolist())
+            nwater = cfg.ocean.nwater[idxWave]
+
+        # incidence angle = phase angle / 2
+        mus = np.cos(sza*dr)
+        muv = np.cos(vza_meshed*dr)
+        xi = np.arccos(mus*muv+np.sin(sza*dr)*np.sin(vza_meshed*dr)*np.cos(phi_meshed*dr))/2.
+        xi = xi/dr
+
+        # specular reflectance
+        ref_sp = ocean_specular_reflectance(xi,nwater)
+
+        # only if a broad spectral domain is considered
+        if do_spectral_averaging == True:
+            ans = np.zeros((1,xi.shape[0]), np.float)
+            for i in range(xi.shape[0]):
+                ans[0,i] = np.average(ref_sp[:,i])
+            ref_sp = ans
+
+        # - BRDF variables
+        m = 1/(np.cos(sza*dr)+np.cos(vza_meshed*dr))
+        Zx = (np.sin(vza_meshed*dr)*np.cos(phi_meshed*dr)+np.sin(sza*dr)) *m
+        Zy = (np.sin(vza_meshed*dr)*np.sin(phi_meshed*dr)) *m
+        tan_beta = np.sqrt(Zx**2+Zy**2)
+        beta = np.arctan(tan_beta)
+
+        vars_ocean['ref_sp'] = ref_sp
+        vars_ocean['tan_beta'] = tan_beta
+        vars_ocean['beta'] = beta
+        vars_ocean['cos_vza'] = cos_vza
+
+    else:
+        ref_sp = vars_ocean['ref_sp']
+        tan_beta = vars_ocean['tan_beta']
+        beta = vars_ocean['beta']
+        cos_vza = vars_ocean['cos_vza']
 
 
-
-    # incidence angle = phase angle / 2
-    mus = np.cos(sza*dr)
-    muv = np.cos(vza_meshed*dr)
-
-    xi = np.arccos(mus*muv+np.sin(sza*dr)*np.sin(vza_meshed*dr)*np.cos(phi_meshed*dr))/2.
-    xi = xi/dr
-    # specular reflectance
-    ref_sp = ocean_specular_reflectance(xi,nwater)
-
-    # only if a broad spectral domain is considered
-    if do_spectral_averaging == True:
-        ans = np.zeros((1,xi.shape[0]), np.float)
-        for i in range(xi.shape[0]):
-            ans[0,i] = np.average(ref_sp[:,i])
-        ref_sp = ans
-
-
-    # - BRDF
-    m = 1/(np.cos(sza*dr)+np.cos(vza_meshed*dr))
-    Zx = (np.sin(vza_meshed*dr)*np.cos(phi_meshed*dr)+np.sin(sza*dr)) *m
-    Zy = (np.sin(vza_meshed*dr)*np.sin(phi_meshed*dr)) *m
-
-    tan_beta = np.sqrt(Zx**2+Zy**2)
-    beta = np.arctan(tan_beta)
+    # - Compute BRDF
     sigma2 = 0.003+5.12*1e-3*ws
     Pws = np.exp(-tan_beta**2/sigma2)/(np.pi*sigma2)
 
-    #if multi_dir == True:
     brdf = np.zeros( ( len(ref.flatten()), len(vza), len(phi) ), np.float)
-    ######### when would this happen vs the other? ###############################
+
+    # Case where len(lmbd) < len(angles) => loop on wavebands
     if  ref_sp.shape[0] < vza.shape[0]:
         for i in range(ref_sp.shape[0]):
-            calc = ref[i] + np.pi*Pws*ref_sp[i,:] / (4*np.cos(beta)**4 * np.cos(vza_meshed*dr) * np.cos(sza*dr) )
+            calc = ref[i] + np.pi*Pws*ref_sp[i,:] / (4*np.cos(beta)**4 * cos_vza * np.cos(sza*dr) )
             # this is reshaped for the cases where calc.shape=(47,1) and brdf[i,:].shape=(47,)
             brdf[i,:,:] = calc.reshape(brdf[i,:,:].shape)
+    # Case where len(lmbd) > len(angles) => loop on angles
     else:
         for i in range(vza.shape[0]):
-            calc = ref + np.pi*Pws[i]*ref_sp[:,i] / (4*np.cos(beta[i])**4 * np.cos(vza_meshed[i]*dr) * np.cos(sza*dr) )
+            calc = ref + np.pi*Pws[i]*ref_sp[:,i] / (4*np.cos(beta[i])**4 * cos_vza[i] * np.cos(sza*dr) )
             # this is reshaped for the cases where calc.shape=(3701,) and brdf[i,:].shape=(3701, 1)
             brdf[:,:,i] = calc.reshape(brdf[:,:,i].shape)
 
-
     return brdf
 
 # END function_brdf_ocean
 # ===================================================================
 
 
-# > MODIF_CB
 # ===================================================================
 # Compute reflectance over Ocean due to foam
 #
@@ -441,15 +518,16 @@ def function_ocean_foam(cfg, ws, lmbd, do_spectral_averaging=False):
           doi:10.1029/2003JC002177
           to determine its spectral dependancy
 
-
     The foam reflectance is assumed isotropic
 
     Inputs
-        - wind_speed [npixels]
+        - cfg parameters
+        - wind_speed:               array [nlon*nlat]
+        - lmbd:                     array [nlmbd]
+        - do_spectral_averaging:    logical
 
     Outputs
-        - reflectance [npixels]
-
+        - ref_foam:                 array [nlon*nlat, nlmbd]
     '''
 
     # determine the spectral dimension
@@ -490,7 +568,7 @@ def function_ocean_foam(cfg, ws, lmbd, do_spectral_averaging=False):
 
 # END function_ocean_foam
 # ===================================================================
-# < MODIF_CB
+
 
 # ===================================================================
 # Compute BRDF over Snow on a pixel basis
@@ -502,17 +580,25 @@ def function_brdf_snow( cfg, sza=None, vza=None, phi=None, lmbd=None,
     Model of A.Kokhanovsky & E.P.Zege (2004, Scattering of snow, Applied
     Optics, 43(7):1589-1602)
 
-    Inputs
+    Inputs:
+        - cfg parameters
         - sun zenith angle (optional):       scalar
-        - view zenith angle (optional):      scalar or vector
-        - relative azimuth angle (optional): scalar or vector
-        - wavelength             : scalar or vector
-        - wavelength for kice  : vector
-        - imaginary part of ice refraction index : vector
+        - view zenith angle (optional):      scalar or array [nvza]
+        - relative azimuth angle (optional): scalar or array [nphi]
+        - lmbd:                              array [nlmbd]
+        - do_spectral_averaging:             logical
+        - fit2obs:                           string
+        - ref_obs:                           array [nlmbd_MODIS = 7]
+        - data_snow dictionnary containing
+           - fveg:                           scalar
+           - snow_grain_size:                scalar
+           - rveg:                           array [nlmbd, 1, 1]
+           - rtot:                           array [nlmbd]
 
-
-    Outputs
-        - reflectance [nlmbd, nvza]
+    Outputs:
+        - dictionnary containing:
+           - refsnow:                        array [nlmbd,nvza,nphi]
+           - data_snow:                      dictionnary
 
     '''
 
@@ -527,9 +613,6 @@ def function_brdf_snow( cfg, sza=None, vza=None, phi=None, lmbd=None,
 
 
     # - Synchronise shapes, this is neccessary because the shape of vza and phi need to always be the same
-    #vza_meshed, phi_meshed = vza, phi
-    #sza_meshed = sza
-    #if vza.shape != phi.shape:
     phi_meshed, vza_meshed = np.meshgrid(phi, vza)
     vza_meshed = vza_meshed.ravel()
     phi_meshed = phi_meshed.ravel()
@@ -544,7 +627,7 @@ def function_brdf_snow( cfg, sza=None, vza=None, phi=None, lmbd=None,
         #  Principles:
         #    1. Determine the fraction of non-snow cover
         #    2. Determine the reflectance associated to non-snow (assuming a linear composition of the 2 components)
-    # if fveg > 0 (we can not neglet non snow elements)
+        # if fveg > 0 (we can not neglet non snow elements)
         #    3. Apply spectral interpolation procedure to non-snow reflectance: rveg
         #    4. Recompute the pseudo-observation of snow reflectance as a function of MODIS observations and previous rveg
         #    5. Fit the snow model at 1240 nm by estimating the snow grain size
@@ -571,13 +654,13 @@ def function_brdf_snow( cfg, sza=None, vza=None, phi=None, lmbd=None,
                 # 2 - 6
                 rveg_obs = (ref_obs - (1-fveg)*rsnow[cfg.indices_modis_bands].ravel())/fveg
                 # 3 - 7
-                rveg = process_reflectance.reflectance_spectrum_land(rveg_obs, cfg.lmbd, cfg)
+                rveg = process_reflectance.reflectance_spectrum_land(rveg_obs, cfg)
 
                 if istep ==0:
                     # 4
                     rsnow_obs_new = (ref_obs-fveg*rveg[cfg.indices_modis_bands])/(1-fveg)
                     snow_grain_size = snow_model(cfg, sza_meshed, vza_meshed, phi_meshed, cfg.lmbd, cfg.land.kice,
-                                                 invert_snow_grain_size = True,
+                                                 do_invert_snow_grain_size = True,
                                                  invert_ref_obs = rsnow_obs_new[4],
                                                  invert_ilmbd = cfg.indices_modis_bands[4])
                     # 5
@@ -590,7 +673,7 @@ def function_brdf_snow( cfg, sza=None, vza=None, phi=None, lmbd=None,
          # 9
         if fveg <0.2:      # only snow
             snow_grain_size = snow_model(cfg, sza_meshed, vza_meshed, phi_meshed, cfg.lmbd, cfg.land.kice,
-                                         invert_snow_grain_size = True,
+                                         do_invert_snow_grain_size = True,
                                          invert_ref_obs = ref_obs[4],
                                          invert_ilmbd = cfg.indices_modis_bands[4])
             rsnow  = snow_model(cfg, sza_meshed, vza_meshed, phi_meshed, cfg.lmbd, cfg.land.kice, snow_grain_size = snow_grain_size).ravel()
@@ -598,11 +681,10 @@ def function_brdf_snow( cfg, sza=None, vza=None, phi=None, lmbd=None,
             ans = rsnow.ravel()
             rveg = rsnow[:]*0
 
-        data_snow['fveg'] = fveg
-        data_snow['snow_grain_size'] = snow_grain_size
-        data_snow['rveg'] = rveg
-        data_snow['rtot'] = ans.ravel()
-
+        data_snow['fveg'] = fveg                         # vegetation fraction
+        data_snow['snow_grain_size'] = snow_grain_size   # snow grain size
+        data_snow['rveg'] = rveg                         # reflectance of vegetation
+        data_snow['rtot'] = ans.ravel()                  # total reflectance
 
     # -
     # - Compute the BRDF
@@ -628,14 +710,6 @@ def function_brdf_snow( cfg, sza=None, vza=None, phi=None, lmbd=None,
             for iwl in range(len(lmbd)): RefSnow[iwl,:,:] = data_snow['rtot'][iwl]
             ans = RefSnow
 
-
-        # - Spectral averaging
-        # if do_spectral_averaging:
-        #     ans  = np.zeros((1,len(vza_meshed)), np.float)
-        #     RefSnow = RefSnow.reshape(nlmbd,len(vza_meshed))
-        #     for i in range(len(vza)):  ans[:,i] = np.average(RefSnow[:,i])
-        #     ans = ans.reshape(1,len(vza),len(phi))
-
     # -
     # - return value
     # -
@@ -651,25 +725,28 @@ def function_brdf_snow( cfg, sza=None, vza=None, phi=None, lmbd=None,
 #
 def snow_model( cfg, sza, vza, phi, lmbd, kice,
                 snow_grain_size = None,
-                invert_snow_grain_size = False, # needed to estimate snow grain size
-                invert_ref_obs         = None,          # needed to estimate snow grain size
-                invert_ilmbd           = None           # needed to estimate snow grain size
+                do_invert_snow_grain_size = False, # needed to estimate snow grain size
+                invert_ref_obs         = None,  # needed to estimate snow grain size
+                invert_ilmbd           = None   # needed to estimate snow grain size
                 ):
 
-    '''Model of A.Kokhanovsky & E.P.Zege (2004, Scattering of snow, Applied
+    '''
+    Model of A.Kokhanovsky & E.P.Zege (2004, Scattering of snow, Applied
     Optics, 43(7):1589-1602)
 
-    Inputs
-    - sun zenith angle:       scalar
-    - view zenith angle:      scalar or vector
-    - relative azimuth angle: scalar or vector
-    - wavelength             : scalar or vector
-    - wavelength for kice  : vector
-    - imaginary part of ice refraction index : vector
+    Inputs:
+        - cfg parameters
+        - sun zenith angle (optional):       scalar
+        - view zenith angle (optional):      scalar or array [nvza]
+        - relative azimuth angle (optional): scalar or array [nphi]
+        - lmbd:                              array [nlmbd]
+        - kice:                              array [nlmbd]
+        - do_invert_snow_grain_size:         logical
+        - invert_ref_obs:                    scalar
+        - invert_ilmbd:                      scalar
 
-
-    Outputs
-        - reflectance [nlmbd, nvza]
+    Outputs:
+        - refsnow:                           array [nlmbd,nvza,nphi]
 
     '''
 
@@ -694,7 +771,7 @@ def snow_model( cfg, sza, vza, phi, lmbd, kice,
 
 
     # - Inverse mode: estimate snow grain size from the measurement at a given wavelength
-    if invert_snow_grain_size == True:
+    if do_invert_snow_grain_size == True:
         alpha = -Ref0*np.log(invert_ref_obs/Ref0)/(Ks*Kv)
         snow_grain_size = alpha*alpha*lmbd[invert_ilmbd]*1e-9/(4.*np.pi*kice[invert_ilmbd]*13)
         return snow_grain_size
@@ -712,15 +789,12 @@ def snow_model( cfg, sza, vza, phi, lmbd, kice,
             RefS[:,i] = Ref0[i]*np.exp(-alpha*Ks[i]*Kv[i]/Ref0[i])
 
 
-    ans = RefS    #.reshape((nlmbd,len(vza),len(phi)))
-
-
     # -
     # - return value
     # -
-    return  ans
+    return  RefS
 
-# END brdf_snow
+# END snow_model
 # ===================================================================
 
 
@@ -728,15 +802,18 @@ def snow_model( cfg, sza, vza, phi, lmbd, kice,
 # ===================================================================
 #  F1 function of the Li-Sparse Reciprocal BRDF model
 #
-#  Inputs:
-#   - sun zenith angle (deg) (scalar or vector)
-#   - view zenith angle (deg) (scalar or vector)
-#   - relative azimuth angle (deg) (scalar or vector)
-#
-# Outputs:
-#   - f1 (scalar or vector)
 # ===================================================================
 def brdf_land_f1(sza,vza,phi):
+    '''
+    F1 function of the Li-Sparse Reciprocal BRDF model
+
+    Inputs:
+        - sun zenith angle (optional):       scalar
+        - view zenith angle (optional):      scalar or array [nvza]
+        - relative azimuth angle (optional): scalar or array [nphi]
+    Outputs:
+        - ans:                               scalar or array [nvza*nphi]
+    '''
 
     dr = np.pi/180
 
@@ -754,24 +831,28 @@ def brdf_land_f1(sza,vza,phi):
 
     temp = (t - sin_t*cos_t)*sec/np.pi - sec
     ans = temp + (1 + cos_ksi)*sec_vza*sec_sza/2.
+
     return np.array(ans)
 
-# END brdf_f1
+# END brdf_land_f1
 # ===================================================================
 
 
 # ===================================================================
 # F2 function of the Roujean + HS model
-#
-# Inputs :
-#  - sun zenith angle (deg) (scalar or vector)
-#  - view zenith angle (deg) (scalar or vector)
-#  - relative azimuth angle (deg) (scalar or vector)
-#
-# Outputs:
-#   - f2 (scalar or vector)
+
 # ===================================================================
 def brdf_land_f2(sza,vza,phi):
+    '''
+    # F2 function of the Roujean + HS Maignan model
+
+    Inputs:
+        - sun zenith angle (optional):       scalar
+        - view zenith angle (optional):      scalar or array [nvza]
+        - relative azimuth angle (optional): scalar or array [nphi]
+    Outputs:
+        - ans:                               scalar or array [nvza*nphi]
+    '''
 
     dr = np.pi/180
 
@@ -781,9 +862,10 @@ def brdf_land_f2(sza,vza,phi):
 
     ans = (2./(3.*np.pi) *(((np.pi - 2.0*ksi)*cos_ksi + 2.0*sin_ksi) /(np.cos(sza*dr) + np.cos(vza*dr))) )* \
           (1 + 1/(1 + ksi/(1.5*np.pi/180))) - 1./3.
+
     return np.array(ans)
 
-# END brdf_f2
+# END brdf_land_f2
 # ===================================================================
 
 
@@ -791,22 +873,23 @@ def brdf_land_f2(sza,vza,phi):
 # ===================================================================
 # Determine vza depending on sza for display
 #
-def define_vza_hotspot(sza, vza):
-    '''Determine vza depending on sza for display.
+def define_vza_hotspot(sza, vza, phi):
+    '''
+    Determine vza depending on sza for display.
 
      Increase resolution around hot spot
 
      Inputs
       - sza  scalar
-      - vza  array of size [nang]
+      - vza  array of size [nvza]
+      - phi scalar
 
      Outputs
-      - vza  array of size [nang+20]
+      - vza  array of size [nvza+20]
     '''
 
-
     # increase vza resolution around hot spot
-    vza_hotspot = np.arange(-10,10,1)+sza
+    vza_hotspot = np.arange(-10,10,1)+np.sign(np.cos(phi*np.pi/180))*sza
     ans = np.sort(np.concatenate((vza,vza_hotspot)))
 
     return ans
@@ -826,18 +909,17 @@ def define_angles_polar_plot( vza, step_polar ):
       - step_polar  scalar
 
      Outputs
-      - vza         array [step_polar,step_polar]
-      - phi         array [step_polar,step_polar]
+      - vza         array [nvza]
+      - phi         array [nphi]
     '''
 
     vza_max = np.amax(vza)
     vza_val = np.linspace(0, vza_max, step_polar)
     phi_val = np.linspace(0, 360, step_polar)
     vza_val,phi_val = np.meshgrid(vza_val,phi_val)
-    #phi_meshed, vza_meshed = np.meshgrid(phi, vza) # CHANGE_CB
 
     return vza_val.ravel(), phi_val.ravel()
-# END define_vza
+# END define_angles_polar_plot
 # ===================================================================
 
 
@@ -856,11 +938,11 @@ def ocean_specular_reflectance( ti, n):
     and with the view zenith angle = solar zenith angle
 
     Inputs
-        - incidence angle :       scalar or vector
-        - water refractive index:      scalar or vector
+        - incidence angle :            scalar or array [nvza*nphi]
+        - water refractive index:      scalar or array [nlmbd]
 
     Outputs
-        - reflectance [len(n)]
+        - rho_sp:                      array [nlmbd, nvza*nphi]
 
     '''
     dr = np.pi/180.
