@@ -29,10 +29,11 @@ import matplotlib
 # need to set 'Agg' as the backend when running by web, this is set in web_run_calc.py
 if os.getenv('ADAM_WEB_REQUEST', '') == 'true':
     matplotlib.use("Agg")
+
 from mpl_toolkits.mplot3d import Axes3D
 import matplotlib.pyplot as plt
 from matplotlib.patches import Circle as circle
-
+from matplotlib.ticker import MultipleLocator
 
 import adam
 
@@ -43,17 +44,6 @@ import adam
 # ===================================================================
 # Display map
 #
-# Inputs:
-#  - map [nlon,nlat] to be displayed
-#  - latitudes (vector of nlat)
-#  - longitudes (vector of nlon)
-#  - enlarge (optional): if True enlarge by offset in map in lat,lon
-# - fig (optional): figure number
-#  - save (optional): save the figure in a file
-#  - save_format (optional): output format of the saved file
-#  - path (optional): output path of the saved file
-#  - filename (optional): output file name
-#
 #
 # - TODO :
 # different colormaps for land and ocean...
@@ -63,6 +53,18 @@ import adam
 def map_plt(data_map, lats, lons, enlarge = False, fig = None,
         save = False, save_format = 'png', path = None, filename = None,
         missval = None, fill_missval = 0):
+    '''
+     Inputs:
+     - data_map [nlon,nlat] to be displayed
+     - latitudes:                vector of nlat
+     - longitudes:               vector of nlon
+     - enlarge (optional):       if True enlarge by offset in map in lat,lon
+     - fig (optional):           figure number
+     - save (optional):          save the figure in a file
+     - save_format (optional):   output format of the saved file
+     - path (optional):          output path of the saved file
+     - filename (optional):      output file name
+     '''
 
     xint = lons[1] - lons[0]
     yint = lats[1] - lats[0]
@@ -135,7 +137,7 @@ def map_plt(data_map, lats, lons, enlarge = False, fig = None,
     plt.subplots_adjust(left=0, right=1, top=1, bottom=0, hspace=0, wspace=0)
 
     if save == True:
-        plt.savefig(filename+'.'+save_format, dpi=72, facecolor='None', edgecolor='None',
+        plt.savefig(filename+'.'+save_format, dpi=72, facecolor='white', edgecolor='None',
                 orientation='portrait', papertype=None, format= save_format,
                 transparent=True, bbox_inches='tight')
     else:
@@ -198,80 +200,105 @@ def map_pil(adam_job, spectral_domain_index=0, width=250, output_dir=None):
     coords_file.write(world_file)
     coords_file.close()
 
-    # mask water
-    #data = np.ma.masked_equal(data, -1)
-
-    # load the default min and max colour ranges obtain by
-    # inspecting the data for the whole globe for each spectral range
-
-    range_dict = {
-      0: {'min': 0, 'max': 1.03365},
-      1: {'min': 0, 'max': 1.03555 },
-      2: {'min': 0, 'max': 1.17559},
-      3: {'min': 0, 'max': 1.41174},
-      4: {'min': 0, 'max': 1.26724}
-    }
-
-
 
+    # get the max and min values for land and ocean from our config file,
+    # these values are normally the 5 and 95 percentile to give the maps the maximum contrast
+    spectral_key = adam_job.spectral_domains[spectral_domain_index]
+    # the config stores the keys as tuples (not a 2-len array) so we need to convert
+    # it to a tuple here to ensure a match
+    spectral_key = tuple(spectral_key)
 
     try:
-        min_value = range_dict[spectral_domain_index]['min']
-        max_value = range_dict[spectral_domain_index]['max']
-    except:
-        min_value = 0
-        max_value = 0.5
-
-
+        land_min = 0
+        land_max  = adam_job.cfg.reflectance_limits[spectral_key]['land']['max']
+        ocean_min = 0
+        ocean_max = adam_job.cfg.reflectance_limits[spectral_key]['ocean']['max']
+    except KeyError:
+        print 'could not find index: %s in the reflectance_limits config item' % [spectral_key]
+        raise
 
     # build our color map lookerupper using matplotlib machinery
     classes = 255
-    mult = classes / (max_value - min_value)
-    cmap = matplotlib.cm.get_cmap('jet', classes)
-    cmap.set_over('r', 1.0)
+    cmap_land = matplotlib.cm.get_cmap('jet', classes)
+    cmap_land.set_over('r', 1.0)
+
+    cmap_ocean = matplotlib.cm.get_cmap('gist_earth', classes)
+    cmap_ocean.set_over('w', 1.0)
 
 
-#
-#    def get_colour_hex(value):
-#        ''' returns the hex string for a particular data value from an existing colour map '''
-#        corrected_value = (value - min_value) * mult
-#        colour = matplotlib.colors.rgb2hex(cmap(int(corrected_value)))
-#        return colour
 
-
-    def get_rgb_tuple(value):
+    def get_rgb_tuple(value, min_value, max_value, cmap):
         ''' returns a r,g,b,a tuple from a particular data value from an existing colour map '''
         if value is np.ma.masked:
             return (0,0,0,0)
 
         # convert the value into a positive number between 0 and number of classes (255)
-        corrected_value = (value - min_value) * mult
+        multiplier = classes / (max_value - min_value)
+        corrected_value = (value - min_value) * multiplier
         # obtain the colour hex code for that value
         hex_code = matplotlib.colors.rgb2hex(cmap(int(corrected_value)))
         # turn the hexcode into rgb values where the values are between 0 and 1
         r,g,b = matplotlib.colors.hex2color(hex_code)
 
         # scale the numbers to between 0 and 255
-        return (r*255, g*255, b*255, 255)
+        #return (r*255, g*255, b*255, 255)
+        return_tuple = tuple( map(int,(r*255, g*255, b*255, 255)) )
+        return return_tuple
 
 
-    # initialise the new image (map)
-    pil_im = Image.new('RGBA', (width, height))
-    # get the image access object
-    pil_acc = pil_im.load()
+#    # initialise the new image (map)
+    pil_im = Image.new('RGBA', (int(width), int(height)))
+#    # get the image access object
+#    pil_acc = pil_im.load()
 
-    # i don't like this loop but for now its like this
-    # loop across the x
-    for i in range(data.shape[0]):
-        # loop across the y
-        for j in range(data.shape[1]):
-            # for each pixel get the rgba value
+    image_array = np.asarray(pil_im).reshape((width*height, 4)).copy()
+    #image_array = np.zeros((width*height, 4))
 
-            pil_acc[i, j] = get_rgb_tuple(data[i,j])
-            #print 'got %s from %s' % (pil_acc[i, j], data[i,j])
+    for index in adam_job.data['idx_land']:
+        image_array[index,:] = get_rgb_tuple( data.flatten()[index],
+                                    land_min,
+                                    land_max,
+                                    cmap_land
+                                )
+
+    for index in adam_job.data['idx_snow']:
+        image_array[index,:] = get_rgb_tuple( data.flatten()[index],
+                                    land_min,
+                                    land_max,
+                                    cmap_land
+                                )
 
 
-    # save the PIL image to disk
+
+    for index in adam_job.data['idx_ocean']:
+        image_array[index,:] = get_rgb_tuple(data.flatten()[index],
+                                    ocean_min,
+                                    ocean_max,
+                                    cmap_ocean
+                                )
+
+
+#    # i don't like this loop but for now its like this
+#    # loop across the x
+#    for i in range(data.shape[0]):
+#        # loop across the y
+#        for j in range(data.shape[1]):
+#            # for each pixel get the rgba value
+#            print get_rgb_tuple(data[i,j])
+#            pil_acc[i, j] = get_rgb_tuple(data[i,j])
+#            #print 'got %s from %s' % (pil_acc[i, j], data[i,j])
+
+
+    image_array = image_array.astype(np.uint8)
+    image_array = image_array.reshape((width, height, 4))
+    #image_array = np.flipud(image_array)
+    #image_array = np.fliplr(image_array)
+    pil_im = Image.fromarray(image_array)
+
+    pil_im = pil_im.transpose(Image.FLIP_LEFT_RIGHT)
+    pil_im = pil_im.transpose(Image.ROTATE_90)
+#    # save the PIL image to disk
+#    print filename
     pil_im.save(filename + '.png', "PNG")
 
 
@@ -289,7 +316,6 @@ def map_pil(adam_job, spectral_domain_index=0, width=250, output_dir=None):
 def map_raw_values(adam_job, spectral_domain_index=0, width=250, output_dir=None):
 
     import osgeo.gdal as gdal
-    import osgeo.osr as osr
 
     # get the data variable
     data = adam_job.data['reflectance_averaged']
@@ -336,14 +362,14 @@ def map_raw_values(adam_job, spectral_domain_index=0, width=250, output_dir=None
     # dodgy scaling hacks
     #data = np.ma.filled(data, 0.)
 
-# reflectance_averaged
-    range_dict = {
-      0: {'min': 0, 'max': 1.03365},
-      1: {'min': 0, 'max': 1.03555 },
-      2: {'min': 0, 'max': 0.842136},
-      3: {'min': 0, 'max': 0.546611},
-      4: {'min': 0, 'max': 0.441895}
-    }
+    # reflectance_averaged
+#    range_dict = {
+#      0: {'min': 0, 'max': 1.03365},
+#      1: {'min': 0, 'max': 1.03555 },
+#      2: {'min': 0, 'max': 0.842136},
+#      3: {'min': 0, 'max': 0.546611},
+#      4: {'min': 0, 'max': 0.441895}
+#    }
     # modis cedric, too light
 #    range_dict = {
 #      0: {'min': 0, 'max': 0.215},
@@ -366,20 +392,22 @@ def map_raw_values(adam_job, spectral_domain_index=0, width=250, output_dir=None
 #    }
 
 
-    this_range = range_dict[spectral_domain_index]['max'] - range_dict[spectral_domain_index]['min']
-    #stretched = data.clip(range_dict[spectral_domain_index]['min'], range_dict[spectral_domain_index]['max'])
-    #stretched = data * 400/range_dict[spectral_domain_index]['max']
-    stretched = data * 255/range_dict[spectral_domain_index]['max']
+    # get the max and min values for land and ocean from our config file,
+    # these values are normally the 5 and 95 percentile to give the maps the maximum contrast
+    spectral_key = adam_job.spectral_domains[spectral_domain_index]
+    spectral_key = tuple(spectral_key)
+
+    try:
+        land_max  = adam_job.cfg.reflectance_limits[spectral_key]['land']['max']
+    except KeyError:
+        print 'could not find index: %s in the reflectance_limits config item' % [spectral_key]
+        raise
+
+
+    # here is the brightness scaling, 255 is 1x brightness, we get best results with ~3x
+    #stretched = data * 255/land_max
+    stretched = data * 650/land_max
     stretched = stretched.clip(0, 255)
-    #stretched = data * (255/range_dict[spectral_domain_index]['max'])
-    #stretched = data
-
-    #print stretched.mask
-    #stretched = np.ma.masked_less_equal(stretched, 0)
-    #stretched = stretched.filled(255)
-
-    #stretched = stretched.astype(np.uint)
-
 
 
     output_driver = gdal.GetDriverByName( "GTiff" )
@@ -457,8 +485,6 @@ def main_ref_spectra(job, case = None, indices=None, title = None):
         x,y = np.meshgrid(job.data['longitude'], job.data['latitude'])
 
         # cycle through each pixel
-
-
         for i in range(len(indices)):
             # load the graphable data
             ref[:,i] = np.ma.array(job.data['BRDF'][indices[i]])
@@ -484,7 +510,7 @@ def main_ref_spectra(job, case = None, indices=None, title = None):
         error_array = error_array[indices,:]
 
         draw_ref_spectra(cfg.lmbd, ref, job, title=title, legend=legend, error_array=error_array,
-                         lmbd_obs = job.cfg.modis_wavebands, ref_obs = ref_obs)
+                         lmbd_obs=job.cfg.modis_wavebands, ref_obs=ref_obs)
 
 
         job.data['BRDF'] = job.data['BRDF'].reshape(original_shape)
@@ -497,7 +523,6 @@ def main_ref_spectra(job, case = None, indices=None, title = None):
     # -
     # - Processing statistics
     # -
-#    if type(data) == type({}):
     elif case == 'stats':
         data = job.data['stats']
         surfaces = data.keys()
@@ -506,16 +531,30 @@ def main_ref_spectra(job, case = None, indices=None, title = None):
             if data[surface] != None:
 
                 fig = plt.figure(figsize=(9,6))
+                ax1 = fig.add_subplot(111)
 
-                plt.errorbar(cfg.lmbd, data[surface]['mean'], yerr = data[surface]['std'],color = '0.8')
+                plt.errorbar(cfg.lmbd, data[surface]['mean'], yerr = data[surface]['std'], label='stdev', color = '0.8')
                 plt.plot(cfg.lmbd, data[surface]['mean'],'r', label = 'mean')
                 plt.plot(cfg.lmbd, data[surface]['min'],'g', label = 'min')
                 plt.plot(cfg.lmbd, data[surface]['max'],'b', label ='max')
-                plt.xlim(400,4000)
+                plt.xlim(300,4000)
+                plt.ylim(0,None)
+                
+                minorLocator = MultipleLocator(100)
+                ax1.xaxis.set_minor_locator(minorLocator)
+
                 plt.xlabel('wavelength (nm)')
                 plt.ylabel('reflectance')
                 plt.legend(loc='upper right')
 
+                # if this job was initialised with wavebands then add them to the graph
+                instrument_wavebands = job.cfg.instrument_wavebands.get(job.instrument_name)
+                if instrument_wavebands is not None:
+                    for band in instrument_wavebands:
+                        xmin = band
+                        xmax = band + 1
+                        plt.axvspan(xmin,xmax, edgecolor='#bbbbbb', facecolor='#bbbbbb')
+
                 extent = '%s,%s %s,%s' % (job.extent['minx'], job.extent['miny'], job.extent['maxx'], job.extent['maxy'])
                 plt.title('Spectrum Graph - %s pixels (%s), Extent: %s' % (surface, data[surface]['npts'], extent))
 
@@ -538,17 +577,21 @@ def main_ref_spectra(job, case = None, indices=None, title = None):
 
                 plt.annotate(annotation, xy=(annotation_x, annotation_y), fontsize=12, xycoords='figure pixels')
 
+                if instrument_wavebands is not None:
+                    annotation = '\nVertical grey lines\nindicate band centres for\ninstrument: %s\n' % job.instrument_name.upper()
+                    annotation += '\nThis does not take into\naccount the instrument \nresponse (spectral responsivity)\n function.'
+                    annotation_y -= 110
+                    plt.annotate(annotation, xy=(annotation_x, annotation_y), fontsize=9, xycoords='figure pixels')
+
+
                 filename = job.build_filename('spectrum_%s' % surface, 'png')
 
-                plt.savefig(filename, dpi=72, facecolor='None', edgecolor='None',
+                plt.savefig(filename, dpi=72, facecolor='white', edgecolor='None',
                             orientation='portrait', papertype=None, format= 'png', bbox_inches=None)
 
                 plt.close(fig)
 
-                job.add_output('brdf_spectrum',
-                               filename,
-                               case='stats',
-                               title='Spectrum Graph - %s' % surface)
+                job.add_output('brdf_spectrum', filename,  case='stats', title='Spectrum Graph - %s' % surface)
 
 
 
@@ -562,16 +605,18 @@ def main_ref_spectra(job, case = None, indices=None, title = None):
 # Input
 #  - data dictionnary containing mean, max, min, and std
 # ===================================================================
-def draw_ref_spectra(x, y, job, title=None, legend=None, yerr=None, error_array=None, lmbd_obs = None, ref_obs = None):
+def draw_ref_spectra(x, y, job, title=None, legend=None, yerr=None, error_array=None, lmbd_obs=None,
+                     ref_obs=None, instrument=None):
 
     colors = ['b', 'g', 'r', 'c', 'm', 'y', 'k']
     nel = y.shape[1]
 
     fig = plt.figure(figsize=(9,6))
-    if title==None:
-        title = ''
-    # plt.plot(x,y)
-    for i in range(nel): plt.plot(x,y[:,i],colors[i])
+    ax1 = fig.add_subplot(111)
+    if title==None:  title = ''
+
+    for i in range(nel):
+        plt.plot(x, y[:,i], colors[i])
 
     if error_array is not None:
         # error array was pixel, spect: we need spect, pixel
@@ -582,12 +627,25 @@ def draw_ref_spectra(x, y, job, title=None, legend=None, yerr=None, error_array=
         plt.plot(x, maximum,'grey')
 
 
-    plt.xlabel('wavelength (nm)')
-    plt.ylabel('reflectance')
+    plt.xlabel('Wavelength (nm)')
+    plt.ylabel('Reflectance')
     plt.legend(legend, loc='upper right')
-    plt.xlim(400,4000)
+    plt.xlim(300, 4000)
+    plt.ylim(0,None)
     plt.title(title)
 
+
+    minorLocator   = MultipleLocator(100)
+    ax1.xaxis.set_minor_locator(minorLocator)
+
+    # if this job was initialised with wavebands then add them to the graph
+    instrument_wavebands = job.cfg.instrument_wavebands.get(job.instrument_name)
+    if instrument_wavebands is not None:
+        for band in instrument_wavebands:
+            xmin =  band
+            xmax = band + 1
+            plt.axvspan(xmin,xmax, edgecolor='#bbbbbb', facecolor='#bbbbbb')
+
     # add MODIS observations
     for i in range(nel):  plt.plot(lmbd_obs,ref_obs[:,i],marker='o',color=colors[i],linestyle='')
 
@@ -606,15 +664,24 @@ def draw_ref_spectra(x, y, job, title=None, legend=None, yerr=None, error_array=
     annotation += 'Lons:\n   %.03f:%.03f\n' % (job.data['longitude'].min(), job.data['longitude'].max())
     annotation += 'SZA: %s\n' % job.sza
     annotation += 'VZA: %s\n' % job.vza
-    annotation += 'phi: %s\n' % job.phi
+    annotation += 'phi: %s\n\n' % job.phi
+
 
     plt.annotate(annotation, xy=(annotation_x, annotation_y), fontsize=12, xycoords='figure pixels')
 
+    if instrument_wavebands is not None:
+        annotation = '\nVertical grey lines\nindicate band centres for\ninstrument: %s\n' % job.instrument_name.upper()
+        annotation += '\nThis does not take into\naccount the instrument \nresponse (spectral responsivity)\n function.'
+        annotation_y -= 90
+        plt.annotate(annotation, xy=(annotation_x, annotation_y), fontsize=9, xycoords='figure pixels')
 
+    annotation_y -= 60
+    annotation = 'Blue points correspond\nto MODIS data in the\nstandard observation\ngeometry'
+    plt.annotate(annotation, xy=(annotation_x, annotation_y), fontsize=9, xycoords='figure pixels')
 
     filename = job.build_filename('spectrum_pixels', 'png')
 
-    plt.savefig(filename, dpi=72, facecolor='None', edgecolor='None',
+    plt.savefig(filename, dpi=72, facecolor='white', edgecolor='None',
                 orientation='portrait', papertype=None, format= 'png', bbox_inches=None)
 
 
@@ -696,15 +763,15 @@ def main_brdf_time(job, case=None):
             plt.plot(x, y, 'o-')
 
             plt.xlabel('')
-            
+
             locs, labels = plt.yticks()
             # if the values are all low (as is the case for ocean pixels) then we
             # present the y axis values in scientific notation
             if locs[-1] < 0.1:
                 plt.subplots_adjust(left=0.15, right=0.8, top=0.95, bottom=0.1)
                 plt.yticks(locs, map(lambda x: "%.4e" % x, locs))
-                
-                
+
+
             plt.ylabel('Reflectance')
             # legend
             leg = plt.legend(legend, loc='upper right')
@@ -716,7 +783,7 @@ def main_brdf_time(job, case=None):
             plt.title(title)
 
             # add the text explaination
-            
+
             annotation_x = 535
             annotation_y = 390
             plt.annotate('ADAM', xy=(annotation_x, annotation_y), fontsize=22, xycoords='figure pixels')
@@ -737,7 +804,7 @@ def main_brdf_time(job, case=None):
 
             # save the graph
             filename = job.build_filename('BRDFtime_pixels_%s' % surface, 'png')
-            plt.savefig(filename, dpi=72, facecolor='None', edgecolor='None',
+            plt.savefig(filename, dpi=72, facecolor='white', edgecolor='None',
                         orientation='portrait', papertype=None, format= 'png', bbox_inches=None)
             plt.close(fig)
 
@@ -762,14 +829,14 @@ def main_brdf_time(job, case=None):
             fig = plt.figure(figsize=(9,6))
             plt.clf()
             plt.subplots_adjust(left=0.1, right=0.8, top=0.95, bottom=0.1)
-            plt.errorbar(x, data[surface]['mean'], yerr = data[surface]['std'], color = '0.8')
+            plt.errorbar(x, data[surface]['mean'], yerr = data[surface]['std'],  label='stdev', color = '0.8')
             plt.plot(x, data[surface]['mean'],'ro-', label = 'mean')
             plt.plot(x, data[surface]['min'], 'go-', label = 'min')
             plt.plot(x, data[surface]['max'], 'bo-', label = 'max')
             plt.xlabel('')
             plt.xticks( np.arange(12), calendar.month_abbr[1:13])
-            
-            
+
+
             locs, labels = plt.yticks()
             # if the values are all low (as is the case for ocean pixels) then we
             # present the y axis values in scientific notation
@@ -792,7 +859,7 @@ def main_brdf_time(job, case=None):
             plt.title('Time Series - %s pixels (max %s), Extent: %s' % (surface, data[surface]['npts'], extent))
 
             # add the text explaination
-            
+
             annotation_x = 535
             annotation_y = 390
             plt.annotate('ADAM', xy=(annotation_x, annotation_y), fontsize=22, xycoords='figure pixels')
@@ -812,7 +879,7 @@ def main_brdf_time(job, case=None):
 
             # save the graph
             filename = job.build_filename('BRDFtime_%s' % surface, 'png')
-            plt.savefig(filename, dpi=72, facecolor='None', edgecolor='None',
+            plt.savefig(filename, dpi=72, facecolor='white', edgecolor='None',
                         orientation='portrait', papertype=None, format= 'png', bbox_inches=None)
             plt.close(fig)
 
@@ -834,7 +901,7 @@ def main_brdf_time(job, case=None):
 #  - viewing zenith angles
 #
 # ===================================================================
-def main_brdf_principal_plane(job, graph_error=False, case=None, indices=None, title=None, spectral_domain_index=0, phi_index=0):
+def main_brdf_transect_plane(job, graph_error=False, case=None, indices=None, title=None, spectral_domain_index=0, phi_index=0):
 
     vza = job.vza
 
@@ -887,7 +954,7 @@ def main_brdf_principal_plane(job, graph_error=False, case=None, indices=None, t
             #txt_lon = u"%4.1f\u00b0" % (data[indices[i]]['lon_user'])
             #legend.extend(['pix %i (%s,%s)'%(indices[i],txt_lat,txt_lon) ])
 
-        draw_brdf_principal_plane(vza, brdf, job, error_array=brdf_err, title=title, legend=legend)
+        draw_brdf_transect_plane(vza, brdf, job, error_array=brdf_err, title=title, legend=legend)
 
     # -
     # - Processing statistics for each surface type
@@ -900,12 +967,8 @@ def main_brdf_principal_plane(job, graph_error=False, case=None, indices=None, t
         for surface in surfaces:
             fig = plt.figure(figsize=(9,6))
 
-            print surface
             if data[surface] != None:
-                print  data[surface]['mean'].shape
-                print  data[surface]['std'].shape
-                print vza.shape
-                plt.errorbar(vza, data[surface]['mean'], yerr = data[surface]['std'],color = '0.8')
+                plt.errorbar(vza, data[surface]['mean'], yerr = data[surface]['std'], label='stdev', color = '0.8')
                 plt.plot(vza, data[surface]['mean'],'r', label = 'mean')
                 plt.plot(vza, data[surface]['min'],'g', label = 'min')
                 plt.plot(vza, data[surface]['max'],'b', label ='max')
@@ -942,23 +1005,23 @@ def main_brdf_principal_plane(job, graph_error=False, case=None, indices=None, t
 
 
                 filename = job.build_filename('BRDFplane_%s' % surface, 'png')
-                plt.savefig(filename, dpi=72, facecolor='None', edgecolor='None',
+                plt.savefig(filename, dpi=72, facecolor='white', edgecolor='None',
                             orientation='portrait', papertype=None, format= 'png', bbox_inches=None)
                 plt.close(fig)
 
 
-                job.add_output('brdf_principle_plane',
+                job.add_output('brdf_transect_plane',
                                filename,
                                case='stats',
-                               title='BRDF Principal Plane - %s' % surface)
+                               title='BRDF Transect Plane - %s' % surface)
 
-# END brdf_principal_plane
+# END brdf_transect_plane
 # ===================================================================
 
 
 
 # ===================================================================
-# Display BRDF in the principal plane
+# Display BRDF in a transect plane
 #
 # Inputs
 #  - data to be displayed (dictionnary)
@@ -966,10 +1029,11 @@ def main_brdf_principal_plane(job, graph_error=False, case=None, indices=None, t
 #  - viewing zenith angles
 #
 # ===================================================================
-def draw_brdf_principal_plane(x, y, job, error_array=None, title=None, legend=None, filename=None):
+def draw_brdf_transect_plane(x, y, job, error_array=None, title=None, legend=None, filename=None):
 
 
     fig = plt.figure(figsize=(9,6))
+
     plt.clf()
     lplt = plt.plot(x,y)
 
@@ -999,6 +1063,8 @@ def draw_brdf_principal_plane(x, y, job, error_array=None, title=None, legend=No
     ltitle = plt.title(title)
 
 
+
+
     plt.subplots_adjust(left=0.1, right=0.8, top=0.95, bottom=0.1)
     annotation_x = 535
     annotation_y = 390
@@ -1017,13 +1083,13 @@ def draw_brdf_principal_plane(x, y, job, error_array=None, title=None, legend=No
 
     filename = job.build_filename('BRDFplane_pixels', 'png')
 
-    plt.savefig(filename, dpi=72, facecolor='None', edgecolor='None',
+    plt.savefig(filename, dpi=72, facecolor='white', edgecolor='None',
                 orientation='portrait', papertype=None, format= 'png', bbox_inches=None)
     plt.close(fig)
 
 
-    job.add_output('brdf_principle_plane', filename, case='pixels', title=title)
-# END brdf_principal_plane
+    job.add_output('brdf_transect_plane', filename, case='pixels', title=title)
+# END brdf_transect_plane
 # ===================================================================
 
 
@@ -1039,20 +1105,20 @@ def draw_brdf_principal_plane(x, y, job, error_array=None, title=None, legend=No
 # TODO : add a colorbar
 #
 # ===================================================================
-def main_brdf_polar_plot(job, case = 'pixels', indices = None, title=None, three_d=False, waveband_index=0):
-    ''' Draw a polar plot of brdf data. 
-    
+def main_brdf_polar_plot(job, case = 'pixels', indices = None, title=None, three_d=False, waveband_index=0, interactive_graph=False):
+    ''' Draw a polar plot of brdf data.
+
     handles the meshing of vza and phi values internally.
-    
+
     calls draw_3d_plot or draw_polar_plot to do the actual work.
-    
+
     '''
-    
+
     # we need vza and phi to be of equal shapes, so we mesh them here.
     phi_meshed, vza_meshed = np.meshgrid(job.phi, job.vza)
     phi_meshed = phi_meshed.ravel()
     vza_meshed = vza_meshed.ravel()
-        
+
     # -
     # - Processing for individual pixels
     # -
@@ -1064,7 +1130,7 @@ def main_brdf_polar_plot(job, case = 'pixels', indices = None, title=None, three
         data = data.reshape(data.shape[0] * data.shape[1], data.shape[2], data.shape[3])
         # data shape is now pixel_index x vza x phi
 
-         
+
         # if we weren't given the indices to graph, assume all
         if indices == None:
             indices = range(len(data))
@@ -1080,7 +1146,7 @@ def main_brdf_polar_plot(job, case = 'pixels', indices = None, title=None, three
         # initialise the graphable data
         brdf = np.zeros((vza_meshed.shape[0], len(indices)))
 
-        
+
         legend = []
         # cycle through each pixel
         for i in range(len(indices)):
@@ -1093,16 +1159,16 @@ def main_brdf_polar_plot(job, case = 'pixels', indices = None, title=None, three
             # build legend text
             legend.extend(['pix %i (%s,%s)'%(indices[i],txt_lat,txt_lon) ])
             label = ' pixel %i (%s,%s)' % (indices[i],txt_lat, txt_lon)
-
+            title = '%s - %s' % (title, i)
 
             if (three_d):
                 filename = job.build_filename('BRDF_3D', 'png', lats=[lat], lons=[lon])
                 draw_3d_plot(data[indices[i]], vza_meshed, phi_meshed, job, label=label,
-                             filename=filename, title='%s - %s' % (title, i))
+                             filename=filename, title=title, interactive_graph=interactive_graph)
             else:
                 filename = job.build_filename('BRDF_polar', 'png', lats=[lat], lons=[lon])
                 draw_polar_plot(data[indices[i]], vza_meshed, phi_meshed, job, label=label,
-                                filename=filename, title='%s - %s' % (title, i))
+                                filename=filename, title=title)
 
 
 
@@ -1122,17 +1188,16 @@ def main_brdf_polar_plot(job, case = 'pixels', indices = None, title=None, three
                 if data[surf] is None: continue
                 brdf = data[surf][var]
                 title = '%s %s: %s (%i pixels)' % (surf.title(), title_string, var.title(), data[surf]['npts'])
-                # the stats function gives us flattened brdf data, we need to reshape that to 
+                # the stats function gives us flattened brdf data, we need to reshape that to
                 # shape: vza, phi
                 brdf = np.reshape(brdf, (len(job.vza), len(job.phi)))
-                
+
                 if (three_d):
                     filename = job.build_filename('BRDF_Stats_3D', 'png')
                     draw_3d_plot(brdf, vza_meshed, phi_meshed, job, filename=filename, title=title)
                 else:
                     filename = job.build_filename('BRDF_Stats', 'png')
                     draw_polar_plot(brdf, vza_meshed, phi_meshed, job, title=title)
-                
 
 
 
@@ -1140,18 +1205,21 @@ def main_brdf_polar_plot(job, case = 'pixels', indices = None, title=None, three
 
 
 
-def draw_3d_plot(brdf, vza_meshed, phi_meshed, job, title=None, filename=None, label=None):
+
+def draw_3d_plot(brdf, vza_meshed, phi_meshed, job, title=None, filename=None, label=None, interactive_graph=False):
     ''' Performs the actual graph for a 3d polar plot.
-    
-    importantly vza_meshed and phi_meshed must be the same shape, 
+
+    importantly vza_meshed and phi_meshed must be the same shape,
     for example created by meshgrid like this:
-        
+
         phi_meshed, vza_meshed = np.meshgrid(job.phi, job.vza)
         phi_meshed = phi_meshed.ravel()
         vza_meshed = vza_meshed.ravel()
-        
+
+    if show_graph is True, the graph is shown using the python interactive graphing,
+    allowing the user to manipulate the graph using the mouse.
     '''
-    
+
     # we need vza and phi to be in the same shape as the brdf array (len(vza) x len(phi))
     vza_meshed = np.reshape( vza_meshed,  brdf.shape )
     phi_meshed = np.reshape( phi_meshed,  brdf.shape )
@@ -1162,17 +1230,22 @@ def draw_3d_plot(brdf, vza_meshed, phi_meshed, job, title=None, filename=None, l
 
     fig = plt.figure(figsize=(9,6))
 
-    # need to allow for the older version of matplotlib on the linux machines
-    ax = plt.gca(projection='3d')
-    #ax = Axes3D(fig)
-    #ax.plot_wireframe(y,x,brdf)
-    ax.plot_surface(y, x, brdf, rstride=1, cstride=1, cmap=matplotlib.cm.jet, linewidth=0.5, antialiased=True, alpha=0.8)
+    # need to detect the version of matplotlib and create the 3d axis accordingly
+    # unfortunately the matplotlib version is a string.
+    if matplotlib.__version__[0] == '1':
+        ax = plt.gca(projection='3d')
+    else:
+        ax = Axes3D(fig)
+        
+    ax.plot_wireframe(y,x,brdf, linewidth=0.5, color='black')
+    ax.plot_surface(y, x, brdf, rstride=1, cstride=1, cmap=matplotlib.cm.jet, linewidth=0, antialiased=True, alpha=0.8)
+    #ax.plot_surface(y, x, brdf, rstride=1, cstride=1, cmap=matplotlib.cm.jet, linewidth=0.5, antialiased=True, alpha=0.8)
 
     ax.set_zlabel('Reflectance')
     # this only works for matplotlib version > 1
     ax.set_title(title)
     if label is not None: ax.set_title('%s: (%s)' % (title, label))
-    
+
     # rotate the view to make it look like the polar graph
     # ax.view_init(elev, azim)
     ax.view_init(30, 5)
@@ -1196,12 +1269,14 @@ def draw_3d_plot(brdf, vza_meshed, phi_meshed, job, title=None, filename=None, l
     if filename is None:
         filename = job.build_filename('BRDF_3D', 'png')
 
-    plt.savefig(filename, dpi=72, facecolor='None', edgecolor='None',
-                orientation='portrait', papertype=None, format= 'png', bbox_inches=None)
-    #plt.close(fig)
-
-
-    job.add_output('brdf_plot_3d', filename, title=title)
+    # if we are asked to make an interactive graph, show it now.
+    if interactive_graph:
+        plt.show()
+    # otherwise save to file and register the product against the job
+    else:
+        plt.savefig(filename, dpi=72, facecolor='white', edgecolor='None',
+                    orientation='portrait', papertype=None, format= 'png', bbox_inches=None)
+        job.add_output('brdf_plot_3d', filename, title=title)
 
 
 
@@ -1218,21 +1293,21 @@ def draw_3d_plot(brdf, vza_meshed, phi_meshed, job, title=None, filename=None, l
 # ===================================================================
 def draw_polar_plot(brdf, vza_meshed, phi_meshed, job, title=None, filename=None, label=None):
     ''' Performs the actual graph for a 2d polar plot.
-    
-    importantly vza_meshed and phi_meshed must be the same shape, 
+
+    importantly vza_meshed and phi_meshed must be the same shape,
     for example created by meshgrid like this:
-        
+
         phi_meshed, vza_meshed = np.meshgrid(job.phi, job.vza)
         phi_meshed = phi_meshed.ravel()
         vza_meshed = vza_meshed.ravel()
-        
+
     '''
 
     fig = plt.figure(figsize=(9,6))
 
     vza_meshed = np.reshape( vza_meshed,  brdf.shape )
     phi_meshed = np.reshape( phi_meshed,  brdf.shape )
-    
+
     # -
     # - Draw contourf
     # -
@@ -1316,7 +1391,7 @@ def draw_polar_plot(brdf, vza_meshed, phi_meshed, job, title=None, filename=None
 
     if filename is None:
         filename = job.build_filename('BRDF_polar', 'png')
-    plt.savefig(filename, dpi=72, facecolor='None', edgecolor='None',
+    plt.savefig(filename, dpi=72, facecolor='white', edgecolor='None',
                 orientation='portrait', papertype=None, format= 'png', bbox_inches=None)
     plt.close(fig)
 
