@@ -1,17 +1,11 @@
 # -*- coding: utf-8 -*-
 """
-title           : brdf_multidir.py
-description     : script using the ADAM Toolkit demonstating taking making BRDF calculations
-                  for multiple phi values
-author          : Ivan.Price@noveltis.fr
-date            : Tue Jul 31 16:55:26 2012
-version         : 0.1
-usage           : python brdf_multidir.py
+title           : brdf_multidir_demo.py
+description     : script using the ADAM Toolkit demonstrating taking making a BRDF
+                  calculation.
 notes           :
 """
 
-
-
 import numpy as np
 import sys
 
@@ -32,6 +26,7 @@ import adam_config as adam_config
 
 # get an instance of an adam 'config' object, this will be attached to any jobs we run
 #my_config = adam_config.adam_config(path_data = my_path_data, output_root_dir = my_output_root_dir)
+print 'loading config...'
 my_config = adam_config.adam_config()
 
 
@@ -40,34 +35,21 @@ my_config = adam_config.adam_config()
 # the 'request_dictionary' from the logfile into this script
 request_dict = { 'fieldOperationType': 'brdf',
                  'fieldSpectralDomain': '860-860 nm',
-                 'fieldBRType': 'principal',
+                 'fieldBRDFType': 'plane',
                  'fieldSunZenith': 0,
                  'fieldViewZenith': 0,
                  'fieldRelAzimuth': 0,
                  'fieldMonth': 'jan',
 
+                 'fieldLatMin'   : 20.3,
+                 'fieldLonMin'   : 0.4,
+                 'fieldLatMax'   : 20.,
+                 'fieldLonMax'   : 0
 
-                # land
-#                 'fieldCorner1Lat'   : 20.3,
-#                 'fieldCorner1Lon'   : 0.4,
-#                 'fieldCorner2Lat'   : 20.,
-#                 'fieldCorner2Lon'   : 0
-
-                # ocean
-#                 'fieldCorner1Lat'   : 0,
-#                 'fieldCorner1Lon'   : 0,
-#                 'fieldCorner2Lat'   : 0.4,
-#                 'fieldCorner2Lon'   : 0.4
-
-                # snow
-                'fieldCorner1Lat'   : 63.3,
-                'fieldCorner1Lon'   : 27.2,
-                'fieldCorner2Lat'   : 63.5,
-                'fieldCorner2Lon'   : 27.6
                 }
 
 # initialise the job, providing us with a blank object ('job') to work with
-print 'init job/config'
+print 'initialise job...'
 job = adam.adam_job( cfg=my_config )
 
 # now validate our request against certain logic rules, and populate any unpopulated required fields
@@ -76,29 +58,49 @@ if not job.validate_input(request_dict):
     raise Exception('request input invalid !')
 
 
-
-
 # now we are ready to load the data from our source database into our job object
 print 'load data...'
 job.load_data()
 
 # print a little output to prove we know where we are in the world
-print "%s land pixels, %s ocean pixels, %s snow pixels" % (len(job.data['idx_land']),
-                                                           len(job.data['idx_ocean']),
-                                                           len(job.data['idx_snow']))
+print '- %s land pixels'  % len(job.data['idx_land'])
+print '- %s ocean pixels' % len(job.data['idx_ocean'])
+print '- %s snow pixels'  % len(job.data['idx_snow'])
+
 
 # call the convenience functions to populate the various variables
 print 'perform reflectance calc...'
 job.process_reflectance()
 
-PHI_VALUES = np.array(range(30, 60))
 
-job.phi = PHI_VALUES
+# phi will be an array of integers from 30 to 60
+job.phi = np.array(range(30, 60))
 
+# vza will be a range with increased resolution around the hotspot
 job.define_vza_hotspot()
 
-print '\n\nperform BRDF calc...'
+print 'perform BRDF calc...'
 job.process_brdf(do_spectral_averaging=True)
-
+print 'saving netcdf BRDF calc...'
 job.save_netcdf()
 
+print 'done.'
+
+#from matplotlib import pylab
+#job.data['BRDF'].shape
+# logitude, latitude, wavelength, vza, phi
+#len(job.data['longitude'])
+#len(job.data['latitude'])
+#job.data['latitude']
+#job.lmbd
+#job.vza
+#job.phi
+#x = job.vza
+#y = job.data['BRDF'][0,0,0,:,0].flatten()
+#pylab.plot(x,y)
+#pylab.show()
+
+
+
+
+
