@@ -22,31 +22,50 @@ import adam
 import adam_config
 
 
-#my_path_data       = '/home/satellites9/cbacour/ADAM/Data/V1/'
-#my_output_root_dir = '/home/satellites9/cbacour/ADAM/OUTPUT/'
+my_output_root_dir = '/home/scratch01/cbacour/ADAM/'
+my_path_data       = '/home/satellites9/cbacour/ADAM/Data/FDS/Outputs/'
+
 
-#my_path_data       = 'p://3895-ADAM/code/data/'
-#my_output_root_dir = 'Y://web-root/adam/web-output/'
 # get an instance of an adam 'config' object, this will be attached to any jobs we run
-#my_config = adam_config.adam_config(path_data = my_path_data, output_root_dir = my_output_root_dir)
+my_config = adam_config.adam_config(path_data = my_path_data, output_root_dir = my_output_root_dir)
+#my_config = adam_config.adam_config()
 
-my_config = adam_config.adam_config()
 
 # define the type of adam calculation we wish to perform..
 # NOTE that you can build a request like this using the web site and copy / paste
 # the 'request_dictionary' from the logfile into this script
-request_dict = {'fieldOperationType': 'spectrum',
-                'fieldMonth'        : 'jan',
-                'fieldSunZenith'    : 40,
-                'fieldViewZenith'   : 30,
-                'fieldRelAzimuth'   : 0,
+request_dict = {'fieldOperationType' : 'spectrum',
+                'fieldMonth'         : 'jan',
+                'fieldSunZenith'     : 45,
+                'fieldViewZenith'    : 0,
+                'fieldRelAzimuth'    : 0,
                 'fieldSpectralDomain': '400-1200 nm',
-
+                'fieldInstrument'    : 'landsat_7',
+                'fieldComputeError': False,
                 # snow with low reflectance values
-                'fieldCorner1Lon': '47.2',
-                'fieldCorner2Lon': '47.3',
-                'fieldCorner2Lat': '52.9',
-                'fieldCorner1Lat': '52.6'
+                # 'fieldLonMin': '47.2',
+                # 'fieldLonMax': '47.3',
+                # 'fieldLatMax': '52.9',
+                # 'fieldLatMin': '52.6'
+
+                # land / ocean pixels
+                #'fieldLatMin'   : 37.2,
+                # 'fieldLonMin'   : -3.7,
+                # 'fieldLatMax'   : 36.4,
+                # 'fieldLonMax'   : -2.9
+                'fieldNetCDF_main' : 'ADAM_netcdf_M01_Lon_3.25_8.85_Lat_21.65_25.25.nc',
+                'fieldNetCDF_covarland' : 'ADAM_netcdf_M01_Lon_3.25_8.85_Lat_21.65_25.25.nc',
+
+                'fieldLatMin'   : 23.,
+                'fieldLonMin'   : 4.1,
+                'fieldLatMax'   : 23.15,
+                'fieldLonMax'   : 4.2
+                
+                # snow flagged
+                 # 'fieldLatMin'   : 76,
+                 # 'fieldLonMin'   : -40.,
+                 # 'fieldLatMax'   : 76.1,
+                 # 'fieldLonMax'   : -40.2
 
                 }
 
@@ -76,6 +95,7 @@ print "%s land pixels, %s ocean pixels, %s snow pixels" % (len(job.data['idx_lan
 
 # call the convenience functions to populate the various variables
 print 'calculate reflectance..'
+
 # populates job.data['reflectance']
 job.process_reflectance()
 
@@ -83,9 +103,8 @@ job.process_reflectance()
 print 'calculate brdf..'
 job.process_brdf(do_spectral_averaging=False)
 
-
-print 'make graphs..'
 # now we can make our graphs
+print 'make graphs..'
 
 # graphs are created differently depending on how many pixels are to be rendered
 # if there are only a few pixels we generate graphs depicting individual pixels
@@ -109,16 +128,5 @@ else:
 print 'saving netcdf..'
 job.save_netcdf()
 
-
-# extra debugging
-#from matplotlib import pylab
-#import numpy as np
-#
-#pylab.plot(job.cfg.lmbd, np.transpose(job.data['reflectance'][0,:,:]))
-#for i in range(len(job.data['idx_snow'])):
-#    pylab.plot(job.cfg.modis_wavebands, job.data['ref_land'][0,i],'o')
-#pylab.show()
-
-
 print 'done.'
 
